% PsychicDilemma.m
%
% RTB wrote it, 18 August 2018. New exercise for QMBC.
% Tip o'the Pin to John K. Kruschke's, "Doing Bayesian Data Analysis"
%
% see also: pNflips2getZheads.m

%% An experiment in telekinesis

% You are a psychologist studying a subtle form of telekinesis that
% involves using thought to bias the outcomes of the flips of a fair coin.
% You instruct a subject to concentrate intensely on "negative thoughts
% about the outcome 'heads'" so that she gets fewer than one would predict
% by chance. Before you begin the experiment, the subject tells you that
% the psychic effort she must exert is extreme, so she will need to stop
% after 25 trials. While you watch, she obtains the following sequence:
% 
%        TTHHTTHTTTHTTTTTTHTTHHTTH
% 
% That is, 8 heads on 25 flips of the coin. We have obtained the coin from
% the National Institute of Standards and Technology, and tested it
% extensively under non-psychic conditions, so we are abolutely certain
% that it is fair (i.e. pHeads = 0.5).
% 
% What is the probability that, out of 25 flips, she obtained 8 or fewer
% heads?
nHeads = 8;
nTrials = 25;
pNull = 0.5;
% With fixed 'N', the appropriate sample space is determined by the
% binomial distribution:
pFixedN = binocdf(nHeads,nTrials,pNull);
% p = 0.0539; Alas, she is not psychic.

%% A last-minute confession:

% After the experiment, however, your subject tells you that she was not
% completely truthful about her 'psychic effort' story. In fact, the only
% way she can exert her powers effectively is if one of her lucky
% numbers is involved. Since 8 is her luckiest number, she had determined
% beforehand that she would stop when she got the 8th head. This just
% happened to occur on the 25th flip.
%
% Does this knowledge change your analysis or your conclusions? Ans.: Yes!
% This is a very different data-generating process and will give us a
% different p-value. We can calculate it by simulation (see below), or we
% can use the negative binomial distribution, y = nbincdf(x,R,p), which
% gives the probability of x-or-fewer failures *before* we get R successes
% given a single-trial probability of success of p. In our case, we want to
% know the probability of 17 or *more* failures before 8 successes. Since
% the function returns x-or-fewer, we need to use 1 - p(x-or-fewer - 1).
% In MATLAB, this would be:
pFixedZ = nbincdf(nTrials-nHeads-1,nHeads,pNull,'upper');
% p = 0.0320; She IS psychic! You begin to write your Nature paper.

% Teaching note: Most of the students will not know about the negative
% binomial distribution, but they can easily do the calculation by
% simulation.

%% Simulation for fixed Z

nSims = 100000;
pHead = 0.5;
N = 25;
zHeads = 8;

% Save the data in two different ways:
% 1. The # of coin flips it took us to get to zHeads on each sim trial:
allN = zeros(nSims,1);
% 2. The proportion of heads (zHeads/nFlips) we got on each sim trial:
allProps = zeros(nSims,1);

for k = 1:nSims
    nFlips = 0;
    nHeads = 0;
    while nHeads < zHeads
        nFlips = nFlips + 1;
        nHeads = nHeads + (rand < pHead);
    end
    allProps(k) = nHeads / nFlips;
    allN(k) = nFlips;
end
pVal = sum(allProps <= (zHeads ./ N)) ./ nSims;

%% Plot the results of our simulation:

figure
subplot(2,1,1)
histogram(allProps);
xlabel('Sample proportion (z/N)');
ylabel('#');
tStr = sprintf('Number of flips to get %d heads if p(Heads) = %.2f',...
    zHeads,pHead);
title(tStr);
ax = axis;
line([zHeads/N,zHeads/N],[ax(3),ax(4)],'Color','r');
pStr = sprintf('p = %.3f',pVal);
text(ax(2)*0.8,ax(4)*0.9,pStr);

subplot(2,1,2)
histogram(allN);
xlabel('# of Flips');
ylabel('#');
title(tStr);
ax = axis;
line([N,N],[ax(3),ax(4)],'Color','r');
text(ax(2)*0.8,ax(4)*0.9,pStr);

%% Other teaching notes:
% 1. This problem, namely that the p-value we calculate depends on our
% stopping intention, has been touted as a fatal flaw in frequentist Null
% Hypothesis Significance Testing (NHST), and used to promote a Bayesian
% approach to statistics. In the latter, the statistician only considers
% the likelihood of the data actually obtained. For coin tosses, this is
% simply (pHeads^z)(1-pHeads)^(N-z), for different values of pHeads.
%
% 2. The frequentist approach does not incorporate any prior information
% about the likelihood that psychic powers actually exist. That is, we are
% treating this experiment with this one subject as if it were the first
% and only such experiment. In reality, we might have a strong prior belief
% that psychic powers don't exist. The Bayesian approach gives us an
% elegant way to combine our prior beliefs with data from an experiment.
%
% 3. The arbitrary nature of using p < 0.05 as a cut-off for Truth. The
% two p-values we obtained are not that different--they just happen to fall
% on opposite sides of the sacred 0.05.